-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- UFPR, BCC, ci210 2013-2 autor: Roberto Hexsel, 26ago2016,23jun2020
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

--++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- testbench para circuitos combinacionais -- multiplexadores
--++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
use work.p_wires.all;
entity tb_mux is
end tb_mux;

architecture TB of tb_mux is

  -- declaracao dos componentes por testar
  component mux2 is
    port(A,B : in  bit; S : in  bit; Z : out bit);
  end component mux2;

  component mux4 is
    port(A,B,C,D : in  bit; S0,S1 : in bit; Z : out bit);
  end component mux4;

  component mux8 is
    port(A,B,C,D,E,F,G,H : in  bit; S0,S1,S2 : in bit; Z : out bit);
  end component mux8;

  component mux8vet is
    port(entr : in  bit_vector; sel: in bit_vector; Z : out bit);
  end component mux8vet;

  
  -- definicao do vetor de testes para MUX-2
  type test_record_2 is record
    s  : bit;       -- entrada de selecao
    a  : reg2;      -- entradas
    mx : bit;       -- saida esperada do MUX
  end record;
  type test_array_2 is array(positive range <>) of test_record_2;

  -- vetor de testes
  constant test_vectors_2 : test_array_2 := (
    --s,   a,   mx
    ('0',b"00",'0'), -- tabela verdade do mux-2
    ('0',b"01",'1'), -- verifique se correta e corrija se for o caso
    ('0',b"10",'0'),
    ('0',b"11",'1'),
    ('1',b"00",'0'),
    ('1',b"01",'1'),
    ('1',b"10",'1'),
    ('1',b"11",'1'),
    -- nao alterar estes tres ultimos --
    ('0',b"11",'1'),
    ('0',b"11",'1'),
    ('0',b"11",'1')
    );

  -- troque a constante para FALSE para testar seus modelos
  constant TST_MUX_2   : boolean := false;
  

  -- definicao do vetor de testes para MUX-4
  type test_record_4 is record
    s  : reg2;      -- entrada de selecao (para todos circuitos)
    a  : reg4;      -- entrada para multiplexadores
    mx : bit;       -- saida esperada do MUX
  end record;
  type test_array_4 is array(positive range <>) of test_record_4;

  -- vetor de testes
  constant test_vectors_4 : test_array_4 := (
    -- s,    a,     mx
    (b"00",b"0000",'0'),
    (b"00",b"0000",'0'),
    (b"00",b"0000",'0'),
    (b"00",b"0000",'0'),
    -- adicionar vetores de teste aqui
    --
    --
    -- nao alterar estes tres ultimos --
    (b"00",b"1111",'1'),
    (b"00",b"1111",'1'),
    (b"00",b"1111",'1')
    );

    -- troque a constante para FALSE para testar seus modelos
  constant TST_MUX_4   : boolean := true;
  
  
  -- definicao do vetor de testes para MUX-8
  type test_record_8 is record
    s  : reg3;      -- entrada de selecao (para todos circuitos)
    a  : reg8;      -- entrada para multiplexadores
    mx : bit;       -- saida esperada do MUX
  end record;
  type test_array_8 is array(positive range <>) of test_record_8;

  -- vetor de testes
  constant test_vectors_8 : test_array_8 := (
    -- s,     a,         mx,
    (b"000",b"00000000",'0'),
    (b"000",b"00000000",'0'),
    (b"000",b"00000000",'0'),
    (b"000",b"00000000",'0'),
    -- adicionar vetores de teste aqui
    --
    --
    -- nao alterar estes tres ultimos --
    (b"000",b"11111111",'1'),
    (b"000",b"11111111",'1'),
    (b"000",b"11111111",'1')
    );

  -- troque a constante para FALSE para testar seus modelos
  constant TST_MUX_8   : boolean := true;
  

  signal s0,s1,s2 : bit := '0';
  signal s : reg3 := "000";
  
  signal saidaMUX2, saidaMUX4, saidaMUX8, saidaMUX8vet, esperadaMUX : bit;
  signal entr_2 : reg2;
  signal entr_4 : reg4;
  signal entr_8 : reg8;

begin

  -- instanciacao dos componentes por testar
  U_mux2:  mux2  port map (entr_2(0), entr_2(1), s0, saidaMUX2);

  U_mux4:  mux4 port map (entr_4(0),entr_4(1),entr_4(2),entr_4(3),
                          s0,s1, saidaMUX4);

  U_mux8:  mux8 port map (entr_8(0),entr_8(1),entr_8(2),entr_8(3),
                          entr_8(4),entr_8(5),entr_8(6),entr_8(7),
                          s0,s1,s2, saidaMUX8);
  
  U_mux8vet:  mux8vet port map (entr_8, s, saidaMUX8vet);
  
  
  -- este processo efetua os testes, executa em paralelo com seus modelos
  U_testValues: process
    variable v_2 : test_record_2;
    variable v_4 : test_record_4;
    variable v_8 : test_record_8;
  begin
    
    -- circuitos de DUAS entradas
    for i in test_vectors_2'range loop
      v_2    := test_vectors_2(i);           -- atribui valores de teste
      s0     <= v_2.s;
      entr_2 <= v_2.a;
      esperadaMUX <= v_2.mx;

      wait for SIM_STEP;                  -- espera propagacao dos sinais

      --------------------------------------------------------------
      -- altere a constante para FALSE para testar
      --------------------------------------------------------------
      assert TST_MUX_2 or (saidaMUX2 = esperadaMUX)
        report LF&LF&  "mux2: saida errada sel="& B2STR(s0) &
        " saiu=" & B2STR(saidaMUX2) & " esperada=" & B2STR(esperadaMUX)&LF
        severity error;

    end loop;  -- DUAS entradas -----------------------------------
      

    -- circuitos de QUATRO entradas -------------------------------
    for i in test_vectors_4'range loop
      v_4 := test_vectors_4(i);           -- atribui valores de teste
      s0     <= v_4.s(0);
      s1     <= v_4.s(1);
      entr_4 <= v_4.a;
      esperadaMUX <= v_4.mx;

      wait for SIM_STEP;                  -- espera propagacao dos sinais

      --------------------------------------------------------------
      -- altere a constante para FALSE para testar
      --------------------------------------------------------------
      assert TST_MUX_4 or (saidaMUX4 = esperadaMUX)
        report LF&LF&  "mux4: saida errada sel="& BV2STR(s(1 downto 0)) &
        " saiu=" & B2STR(saidaMUX4) & " esperada=" & B2STR(esperadaMUX)&LF
        severity error;

    end loop;  -- QUATRO entradas ---------------------------------
      

    -- circuitos de OITO entradas ---------------------------------
    for i in test_vectors_8'range loop
      v_8    := test_vectors_8(i);      -- atribui valores de teste
      s      <= v_8.s;
      s0     <= v_8.s(0);
      s1     <= v_8.s(1);
      s2     <= v_8.s(2);
      entr_8 <= v_8.a;
      esperadaMUX <= v_8.mx;

      wait for SIM_STEP;               -- espera propagacao dos sinais

      --------------------------------------------------------------
      -- altere a constante para FALSE para testar
      --------------------------------------------------------------
      assert TST_MUX_8 or (saidaMUX8 = esperadaMUX)
        report LF&LF&  "mux8: saida errada sel="& BV2STR(s) &
        " saiu=" & B2STR(saidaMUX8) & " esperada=" & B2STR(esperadaMUX)&LF
        severity error;

      --------------------------------------------------------------
      -- altere a constante para FALSE para testar
      --------------------------------------------------------------
      assert TST_MUX_8 or (saidaMUX8vet = esperadaMUX)
        report LF&LF&  "mux8vet: saida errada sel="& BV2STR(s) &
        " saiu=" & B2STR(saidaMUX8vet) & " esperada=" & B2STR(esperadaMUX)&LF
        severity error;
          
    end loop;  -- OITO entradas ----------------------------------

    wait;
    
    end process U_testValues;

end architecture TB;
-- --------------------------------------------------------------

-- --------------------------------------------------------------
configuration CFG_TB of TB_MUX is
        for TB
        end for;
end CFG_TB;
-- --------------------------------------------------------------

